from pathlib import Path
from os import listdir


def read_meta_file(file_name):
    with open(file_name) as meta_file:
        return dict([line.strip().split("=") for line in meta_file])


def trace_files_from_dir(directory):
    # find meta info
    file_names = listdir(directory)
    meta_file = [f for f in file_names if "meta" in f]

    if not meta_file:
        raise ValueError("Could not find a meta file in given directory.")

    meta_info = read_meta_file(Path(directory) / meta_file[0])
    
    # filter out any non-binary files
    prefix = meta_info["fileprefix"]
    traces = [f for f in file_names if f.startswith(prefix) and f.endswith(".bin")]

    # abort if there are files missing
    num_procs = int(meta_info["numprocs"])
    if len(traces) != num_procs:
        raise ValueError("Some binary traces are missing.")

    # sort by proc id and prepend directory
    traces = sorted(traces, key=lambda t: int(t[len(prefix)+1:len(t)-len(".bin")]))
    return [Path(directory) / trace for trace in traces]



from enum import IntEnum


class DataType(IntEnum):
    ERROR = 0
    NULL = 1
    CHAR = 2
    SIGNED_CHAR = 3
    UNSIGNED_CHAR = 4
    BYTE = 5
    WCHAR = 6
    SHORT = 7
    UNSIGNED_SHORT = 8
    INT = 9
    UNSIGNED = 10
    LONG = 11
    UNSIGNED_LONG = 12
    FLOAT = 13
    DOUBLE = 14
    LONG_DOUBLE = 15
    LONG_LONG_INT = 16
    UNSIGNED_LONG_LONG = 17
    LONG_LONG = 18
    PACKED = 19
    LB = 20
    UB = 21
    FLOAT_INT = 22
    DOUBLE_INT = 23
    LONG_INT = 24
    SHORT_INT = 25
    TWO_INT = 26
    LONG_DOUBLE_INT = 27



from ctypes import c_int, c_void_p, c_ushort, Structure, POINTER, CFUNCTYPE
from .dtypes import DumpiTime, DumpiPerfInfo


CALLBACK = CFUNCTYPE(c_int, c_void_p, c_ushort, POINTER(DumpiTime), 
    POINTER(DumpiTime), POINTER(DumpiPerfInfo), c_void_p)

class DumpiCallbacks(Structure):
    _fields_ = [
        ("on_send", CALLBACK),
        ("on_recv", CALLBACK),
        ("on_get_count", CALLBACK),
        ("on_bsend", CALLBACK),
        ("on_ssend", CALLBACK),
        ("on_rsend", CALLBACK),
        ("on_buffer_attach", CALLBACK),
        ("on_buffer_detach", CALLBACK),
        ("on_isend", CALLBACK),
        ("on_ibsend", CALLBACK),
        ("on_issend", CALLBACK),
        ("on_irsend", CALLBACK),
        ("on_irecv", CALLBACK),
        ("on_wait", CALLBACK),
        ("on_test", CALLBACK),
        ("on_request_free", CALLBACK),
        ("on_waitany", CALLBACK),
        ("on_testany", CALLBACK),
        ("on_waitall", CALLBACK),
        ("on_testall", CALLBACK),
        ("on_waitsome", CALLBACK),
        ("on_testsome", CALLBACK),
        ("on_iprobe", CALLBACK),
        ("on_probe", CALLBACK),
        ("on_cancel", CALLBACK),
        ("on_test_cancelled", CALLBACK),
        ("on_send_init", CALLBACK),
        ("on_bsend_init", CALLBACK),
        ("on_ssend_init", CALLBACK),
        ("on_rsend_init", CALLBACK),
        ("on_recv_init", CALLBACK),
        ("on_start", CALLBACK),
        ("on_startall", CALLBACK),
        ("on_sendrecv", CALLBACK),
        ("on_sendrecv_replace", CALLBACK),
        ("on_type_contiguous", CALLBACK),
        ("on_type_vector", CALLBACK),
        ("on_type_hvector", CALLBACK),
        ("on_type_indexed", CALLBACK),
        ("on_type_hindexed", CALLBACK),
        ("on_type_struct", CALLBACK),
        ("on_address", CALLBACK),
        ("on_type_extent", CALLBACK),
        ("on_type_size", CALLBACK),
        ("on_type_lb", CALLBACK),
        ("on_type_ub", CALLBACK),
        ("on_type_commit", CALLBACK),
        ("on_type_free", CALLBACK),
        ("on_get_elements", CALLBACK),
        ("on_pack", CALLBACK),
        ("on_unpack", CALLBACK),
        ("on_pack_size", CALLBACK),
        ("on_barrier", CALLBACK),
        ("on_bcast", CALLBACK),
        ("on_gather", CALLBACK),
        ("on_gatherv", CALLBACK),
        ("on_scatter", CALLBACK),
        ("on_scatterv", CALLBACK),
        ("on_allgather", CALLBACK),
        ("on_allgatherv", CALLBACK),
        ("on_alltoall", CALLBACK),
        ("on_alltoallv", CALLBACK),
        ("on_reduce", CALLBACK),
        ("on_op_create", CALLBACK),
        ("on_op_free", CALLBACK),
        ("on_allreduce", CALLBACK),
        ("on_reduce_scatter", CALLBACK),
        ("on_scan", CALLBACK),
        ("on_group_size", CALLBACK),
        ("on_group_rank", CALLBACK),
        ("on_group_translate_ranks", CALLBACK),
        ("on_group_compare", CALLBACK),
        ("on_comm_group", CALLBACK),
        ("on_group_union", CALLBACK),
        ("on_group_intersection", CALLBACK),
        ("on_group_difference", CALLBACK),
        ("on_group_incl", CALLBACK),
        ("on_group_excl", CALLBACK),
        ("on_group_range_incl", CALLBACK),
        ("on_group_range_excl", CALLBACK),
        ("on_group_free", CALLBACK),
        ("on_comm_size", CALLBACK),
        ("on_comm_rank", CALLBACK),
        ("on_comm_compare", CALLBACK),
        ("on_comm_dup", CALLBACK),
        ("on_comm_create", CALLBACK),
        ("on_comm_split", CALLBACK),
        ("on_comm_free", CALLBACK),
        ("on_comm_test_inter", CALLBACK),
        ("on_comm_remote_size", CALLBACK),
        ("on_comm_remote_group", CALLBACK),
        ("on_intercomm_create", CALLBACK),
        ("on_intercomm_merge", CALLBACK),
        ("on_keyval_create", CALLBACK),
        ("on_keyval_free", CALLBACK),
        ("on_attr_put", CALLBACK),
        ("on_attr_get", CALLBACK),
        ("on_attr_delete", CALLBACK),
        ("on_topo_test", CALLBACK),
        ("on_cart_create", CALLBACK),
        ("on_dims_create", CALLBACK),
        ("on_graph_create", CALLBACK),
        ("on_graphdims_get", CALLBACK),
        ("on_graph_get", CALLBACK),
        ("on_cartdim_get", CALLBACK),
        ("on_cart_get", CALLBACK),
        ("on_cart_rank", CALLBACK),
        ("on_cart_coords", CALLBACK),
        ("on_graph_neighbors_count", CALLBACK),
        ("on_graph_neighbors", CALLBACK),
        ("on_cart_shift", CALLBACK),
        ("on_cart_sub", CALLBACK),
        ("on_cart_map", CALLBACK),
        ("on_graph_map", CALLBACK),
        ("on_get_processor_name", CALLBACK),
        ("on_get_version", CALLBACK),
        ("on_errhandler_create", CALLBACK),
        ("on_errhandler_set", CALLBACK),
        ("on_errhandler_get", CALLBACK),
        ("on_errhandler_free", CALLBACK),
        ("on_error_string", CALLBACK),
        ("on_error_class", CALLBACK),
        ("on_wtime", CALLBACK),
        ("on_wtick", CALLBACK),
        ("on_init", CALLBACK),
        ("on_finalize", CALLBACK),
        ("on_initialized", CALLBACK),
        ("on_abort", CALLBACK),
        ("on_close_port", CALLBACK),
        ("on_comm_accept", CALLBACK),
        ("on_comm_connect", CALLBACK),
        ("on_comm_disconnect", CALLBACK),
        ("on_comm_get_parent", CALLBACK),
        ("on_comm_join", CALLBACK),
        ("on_comm_spawn", CALLBACK),
        ("on_comm_spawn_multiple", CALLBACK),
        ("on_lookup_name", CALLBACK),
        ("on_open_port", CALLBACK),
        ("on_publish_name", CALLBACK),
        ("on_unpublish_name", CALLBACK),
        ("on_accumulate", CALLBACK),
        ("on_get", CALLBACK),
        ("on_put", CALLBACK),
        ("on_win_complete", CALLBACK),
        ("on_win_create", CALLBACK),
        ("on_win_fence", CALLBACK),
        ("on_win_free", CALLBACK),
        ("on_win_get_group", CALLBACK),
        ("on_win_lock", CALLBACK),
        ("on_win_post", CALLBACK),
        ("on_win_start", CALLBACK),
        ("on_win_test", CALLBACK),
        ("on_win_unlock", CALLBACK),
        ("on_win_wait", CALLBACK),
        ("on_alltoallw", CALLBACK),
        ("on_exscan", CALLBACK),
        ("on_add_error_class", CALLBACK),
        ("on_add_error_code", CALLBACK),
        ("on_add_error_string", CALLBACK),
        ("on_comm_call_errhandler", CALLBACK),
        ("on_comm_create_keyval", CALLBACK),
        ("on_comm_delete_attr", CALLBACK),
        ("on_comm_free_keyval", CALLBACK),
        ("on_comm_get_attr", CALLBACK),
        ("on_comm_get_name", CALLBACK),
        ("on_comm_set_attr", CALLBACK),
        ("on_comm_set_name", CALLBACK),
        ("on_file_call_errhandler", CALLBACK),
        ("on_grequest_complete", CALLBACK),
        ("on_grequest_start", CALLBACK),
        ("on_init_thread", CALLBACK),
        ("on_is_thread_main", CALLBACK),
        ("on_query_thread", CALLBACK),
        ("on_status_set_cancelled", CALLBACK),
        ("on_status_set_elements", CALLBACK),
        ("on_type_create_keyval", CALLBACK),
        ("on_type_delete_attr", CALLBACK),
        ("on_type_dup", CALLBACK),
        ("on_type_free_keyval", CALLBACK),
        ("on_type_get_attr", CALLBACK),
        ("on_type_get_contents", CALLBACK),
        ("on_type_get_envelope", CALLBACK),
        ("on_type_get_name", CALLBACK),
        ("on_type_set_attr", CALLBACK),
        ("on_type_set_name", CALLBACK),
        ("on_type_match_size", CALLBACK),
        ("on_win_call_errhandler", CALLBACK),
        ("on_win_create_keyval", CALLBACK),
        ("on_win_delete_attr", CALLBACK),
        ("on_win_free_keyval", CALLBACK),
        ("on_win_get_attr", CALLBACK),
        ("on_win_get_name", CALLBACK),
        ("on_win_set_attr", CALLBACK),
        ("on_win_set_name", CALLBACK),
        ("on_alloc_mem", CALLBACK),
        ("on_comm_create_errhandler", CALLBACK),
        ("on_comm_get_errhandler", CALLBACK),
        ("on_comm_set_errhandler", CALLBACK),
        ("on_file_create_errhandler", CALLBACK),
        ("on_file_get_errhandler", CALLBACK),
        ("on_file_set_errhandler", CALLBACK),
        ("on_finalized", CALLBACK),
        ("on_free_mem", CALLBACK),
        ("on_get_address", CALLBACK),
        ("on_info_create", CALLBACK),
        ("on_info_delete", CALLBACK),
        ("on_info_dup", CALLBACK),
        ("on_info_free", CALLBACK),
        ("on_info_get", CALLBACK),
        ("on_info_get_nkeys", CALLBACK),
        ("on_info_get_nthkey", CALLBACK),
        ("on_info_get_valuelen", CALLBACK),
        ("on_info_set", CALLBACK),
        ("on_pack_external", CALLBACK),
        ("on_pack_external_size", CALLBACK),
        ("on_request_get_status", CALLBACK),
        ("on_type_create_darray", CALLBACK),
        ("on_type_create_hindexed", CALLBACK),
        ("on_type_create_hvector", CALLBACK),
        ("on_type_create_indexed_block", CALLBACK),
        ("on_type_create_resized", CALLBACK),
        ("on_type_create_struct", CALLBACK),
        ("on_type_create_subarray", CALLBACK),
        ("on_type_get_extent", CALLBACK),
        ("on_type_get_true_extent", CALLBACK),
        ("on_unpack_external", CALLBACK),
        ("on_win_create_errhandler", CALLBACK),
        ("on_win_get_errhandler", CALLBACK),
        ("on_win_set_errhandler", CALLBACK),
        ("on_file_open", CALLBACK),
        ("on_file_close", CALLBACK),
        ("on_file_delete", CALLBACK),
        ("on_file_set_size", CALLBACK),
        ("on_file_preallocate", CALLBACK),
        ("on_file_get_size", CALLBACK),
        ("on_file_get_group", CALLBACK),
        ("on_file_get_amode", CALLBACK),
        ("on_file_set_info", CALLBACK),
        ("on_file_get_info", CALLBACK),
        ("on_file_set_view", CALLBACK),
        ("on_file_get_view", CALLBACK),
        ("on_file_read_at", CALLBACK),
        ("on_file_read_at_all", CALLBACK),
        ("on_file_write_at", CALLBACK),
        ("on_file_write_at_all", CALLBACK),
        ("on_file_iread_at", CALLBACK),
        ("on_file_iwrite_at", CALLBACK),
        ("on_file_read", CALLBACK),
        ("on_file_read_all", CALLBACK),
        ("on_file_write", CALLBACK),
        ("on_file_write_all", CALLBACK),
        ("on_file_iread", CALLBACK),
        ("on_file_iwrite", CALLBACK),
        ("on_file_seek", CALLBACK),
        ("on_file_get_position", CALLBACK),
        ("on_file_get_byte_offset", CALLBACK),
        ("on_file_read_shared", CALLBACK),
        ("on_file_write_shared", CALLBACK),
        ("on_file_iread_shared", CALLBACK),
        ("on_file_iwrite_shared", CALLBACK),
        ("on_file_read_ordered", CALLBACK),
        ("on_file_write_ordered", CALLBACK),
        ("on_file_seek_shared", CALLBACK),
        ("on_file_get_position_shared", CALLBACK),
        ("on_file_read_at_all_begin", CALLBACK),
        ("on_file_read_at_all_end", CALLBACK),
        ("on_file_write_at_all_begin", CALLBACK),
        ("on_file_write_at_all_end", CALLBACK),
        ("on_file_read_all_begin", CALLBACK),
        ("on_file_read_all_end", CALLBACK),
        ("on_file_write_all_begin", CALLBACK),
        ("on_file_write_all_end", CALLBACK),
        ("on_file_read_ordered_begin", CALLBACK),
        ("on_file_read_ordered_end", CALLBACK),
        ("on_file_write_ordered_begin", CALLBACK),
        ("on_file_write_ordered_end", CALLBACK),
        ("on_file_get_type_extent", CALLBACK),
        ("on_register_datarep", CALLBACK),
        ("on_file_set_atomicity", CALLBACK),
        ("on_file_get_atomicity", CALLBACK),
        ("on_file_sync", CALLBACK),
        ("on_iotest", CALLBACK),
        ("on_iowait", CALLBACK),
        ("on_iotestall", CALLBACK),
        ("on_iowaitall", CALLBACK),
        ("on_iotestany", CALLBACK),
        ("on_iowaitany", CALLBACK),
        ("on_iowaitsome", CALLBACK),
        ("on_iotestsome", CALLBACK),
        ("on_function_enter", CALLBACK),
        ("on_function_exit", CALLBACK)
    ]



import os
import sys
import setuptools
from subprocess import call
from ctypes.util import find_library
from setuptools.dist import Distribution
from setuptools.command.build_py import build_py


class BinaryDistribution(Distribution):

    def is_pure(self):
        return False

    def root_is_pure(self):
        return False


class BuildSharedLib(build_py):

    def run(self):
        if not self.dry_run:

            # find libundumpi and build it, if not installed
            lib = find_library("undumpi")
            if not lib or "libundumpi.so" not in lib:
                cwd = os.getcwd()
                repo_url = "https://github.com/sstsimulator/sst-dumpi"
                commit_hash = "5744b8d8f8372171c81f55be43ed658bde6a58fa"
                call(["git", "clone", repo_url])

                os.chdir("sst-dumpi")
                call(["git", "checkout", commit_hash])
                call(["./bootstrap.sh"])
                call(["./configure", "--disable-static", "--prefix={0}/pydumpi".format(cwd)])
                call(["make"])
                call(["make", "install"])
                os.chdir(cwd)

        build_py.run(self)


with open("README.md", "r") as f:
    long_description = f.read()

setuptools.setup(
    name="pydumpi",
    version="0.1.1",
    author="Tobias Schwackenhofer",
    author_email="tobiasschw@gmail.com",
    description="Python bindings for the sst-dumpi trace format.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/justacid/pydumpi",
    distclass=BinaryDistribution,
    packages=["pydumpi"],
    package_data={"pydumpi": ["lib/libundumpi.so.8.0.0"]},
    python_requires=">=3.4",
    cmdclass={"build_py": BuildSharedLib},
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    )
)



from ctypes import *


class DumpiHeader(Structure):
    _fields_ = [
        ("version", c_byte*3),
        ("starttime", c_ulonglong),
        ("hostname", c_char_p),
        ("username", c_char_p),
        ("meshdim", c_int),
        ("meshcrd", POINTER(c_int)),
        ("meshsize", POINTER(c_int)),
    ]

class DumpiFooter(Structure):
    _fields_ = [
        ("call_count", c_int*291),
        ("ignored_count", c_int*291),
    ]

class DumpiKeyvalEntry(Structure):
    pass

# need to separate the fields here, to allow incomplete type
DumpiKeyvalEntry._fields_ = [
    ("key", c_char_p),
    ("val", c_char_p),
    ("next", POINTER(DumpiKeyvalEntry)),
]

class DumpiKeyvalRecord(Structure):
    _fields_ = [
        ("count", c_int),
        ("head", POINTER(DumpiKeyvalEntry)),
        ("tail", POINTER(DumpiKeyvalEntry)),
    ]

class DumpiSizeof(Structure):
    _fields_ = [
        ("count", c_short),
        ("size", POINTER(c_int)),
    ]

class DumpiClock(Structure):
    _fields_ = [
        ("sec", c_int),
        ("nsec", c_int),
    ]

    def __str__(self):
        return "DumpiClock(sec={0}, nsec={1})]".format(self.sec, self.nsec)

    def to_sec(self):
        return self.sec + self.nsec / 1e9

    def to_ms(self):
        return self.sec * 1000 + self.nsec * 1e-6

    def to_ns(self):
        return int(self.nsec + self.sec * 1e9)

    def __sub__(self, rhs):
        nsec = (self.nsec + self.sec * 1e9) - (rhs.nsec + rhs.sec * 1e9)
        result = DumpiClock(int(nsec / 1e9), int(nsec % 1e9)) 
        if nsec < 0:
            result.nsec = abs(result.nsec) \
                if result.sec != 0 else -abs(result.nsec)
        return result

    def __add__(self, rhs):
        nsec = (self.nsec + self.sec * 1e9) + (rhs.nsec + rhs.sec * 1e9)
        result = DumpiClock(int(nsec / 1e9), int(nsec % 1e9)) 
        if nsec < 0:
            result.nsec = abs(result.nsec) \
                if result.sec != 0 else -abs(result.nsec)
        return result

class DumpiTime(Structure):
    _fields_ = [
        ("start", DumpiClock),
        ("stop", DumpiClock),
    ]

class DumpiPerfInfo(Structure):
    _fields_ = [
        ("count", c_int),
        ("counter_tag", (c_char * 128) * 80),
        ("invalue", c_longlong * 128),
        ("outvalue", c_longlong * 128),
    ]

class File(Structure):
    _fields_ = [
    ]

class DumpiMemoryBuffer(Structure):
    _fields_ = [
        ("length", c_size_t),
        ("pos", c_size_t),
        ("buffer", POINTER(c_ubyte)),
    ]

class DumpiProfile(Structure):
    _fields_ = [
        ("file", POINTER(File)),
        ("cpu_time_offset", c_int),
        ("wall_time_offset", c_int),
        ("header", c_long),
        ("body", c_long),
        ("footer", c_long),
        ("keyval", c_long),
        ("perflbl", c_long),
        ("addrlbl", c_long),
        ("sizelbl", c_long),
        ("version", c_char*3),
        ("membuf", POINTER(DumpiMemoryBuffer)),
        ("target_membuf_size", c_size_t),
    ]

class DumpiStatus(Structure):
    _fields_ = [
        ("bytes", c_int),
        ("source", c_int),
        ("tag", c_int),
        ("cancelled", c_byte),
        ("error", c_byte),
    ]

class DumpiFuncCall(Structure):
    _fields_ = [
        ("fn", c_longlong),
    ]

class DumpiSend(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
    ]

class DumpiRecv(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("source", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiGetCount(Structure):
    _fields_ = [
        ("status", POINTER(DumpiStatus)),
        ("datatype", c_short),
        ("count", c_int),
    ]

class DumpiBSend(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
    ]

class DumpiSSend(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
    ]

class DumpiRSend(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
    ]

class DumpiBufferAttach(Structure):
    _fields_ = [
        ("size", c_int),
    ]

class DumpiBufferDetach(Structure):
    _fields_ = [
        ("size", c_int),
    ]

class DumpiISend(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiIbSend(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiIsSend(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiIrSend(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiIRecv(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("source", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiWait(Structure):
    _fields_ = [
        ("request", c_int),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiTest(Structure):
    _fields_ = [
        ("request", c_int),
        ("flag", c_int),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiRequestFree(Structure):
    _fields_ = [
        ("request", c_int),
    ]

class DumpiWaitAny(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("index", c_int),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiTestAny(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("index", c_int),
        ("flag", c_int),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiWaitAll(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("statuses", POINTER(DumpiStatus)),
    ]

class DumpiTestAll(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("flag", c_int),
        ("statuses", POINTER(DumpiStatus)),
    ]

class DumpiWaitSome(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("outcount", c_int),
        ("indices", POINTER(c_int)),
        ("statuses", POINTER(DumpiStatus)),
    ]

class DumpiTestSome(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("outcount", c_int),
        ("indices", POINTER(c_int)),
        ("statuses", POINTER(DumpiStatus)),
    ]

class DumpiIProbe(Structure):
    _fields_ = [
        ("source", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("flag", c_int),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiProbe(Structure):
    _fields_ = [
        ("source", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiCancel(Structure):
    _fields_ = [
        ("request", c_int),
    ]

class DumpiTestCancelled(Structure):
    _fields_ = [
        ("status", POINTER(DumpiStatus)),
        ("cancelled", c_int),
    ]

class DumpiSendInit(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiBsendInit(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiSsendInit(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiRsendInit(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiRecvInit(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("source", c_int),
        ("tag", c_int),
        ("comm", c_short),
        ("request", c_int),
    ]

class DumpiStart(Structure):
    _fields_ = [
        ("request", c_int),
    ]

class DumpiStartall(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
    ]

class DumpiSendrecv(Structure):
    _fields_ = [
        ("sendcount", c_int),
        ("sendtype", c_short),
        ("dest", c_int),
        ("sendtag", c_int),
        ("recvcount", c_int),
        ("recvtype", c_short),
        ("source", c_int),
        ("recvtag", c_int),
        ("comm", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiSendrecvReplace(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("dest", c_int),
        ("sendtag", c_int),
        ("source", c_int),
        ("recvtag", c_int),
        ("comm", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiTypeContiguous(Structure):
    _fields_ = [
        ("count", c_int),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeVector(Structure):
    _fields_ = [
        ("count", c_int),
        ("blocklength", c_int),
        ("stride", c_int),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeHvector(Structure):
    _fields_ = [
        ("count", c_int),
        ("blocklength", c_int),
        ("stride", c_int),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeIndexed(Structure):
    _fields_ = [
        ("count", c_int),
        ("lengths", POINTER(c_int)),
        ("indices", POINTER(c_int)),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeHindexed(Structure):
    _fields_ = [
        ("count", c_int),
        ("lengths", POINTER(c_int)),
        ("indices", POINTER(c_int)),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeStruct(Structure):
    _fields_ = [
        ("count", c_int),
        ("lengths", POINTER(c_int)),
        ("indices", POINTER(c_int)),
        ("oldtypes", POINTER(c_short)),
        ("newtype", c_short),
    ]

class DumpiAddress(Structure):
    _fields_ = [
        ("address", c_int),
    ]

class DumpiTypeExtent(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("extent", c_int),
    ]

class DumpiTypeSize(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("size", c_int),
    ]

class DumpiTypeLb(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("lb", c_int),
    ]

class DumpiTypeUb(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("ub", c_int),
    ]

class DumpiTypeCommit(Structure):
    _fields_ = [
        ("datatype", c_short),
    ]

class DumpiTypeFree(Structure):
    _fields_ = [
        ("datatype", c_short),
    ]

class DumpiGetElements(Structure):
    _fields_ = [
        ("status", POINTER(DumpiStatus)),
        ("datatype", c_short),
        ("elements", c_int),
    ]

class DumpiPack(Structure):
    _fields_ = [
        ("incount", c_int),
        ("datatype", c_short),
        ("outcount", c_int),
        ("pos_in", c_int),
        ("pos_out", c_int),
        ("comm", c_short),
    ]

class DumpiUnpack(Structure):
    _fields_ = [
        ("incount", c_int),
        ("pos_in", c_int),
        ("pos_out", c_int),
        ("outcount", c_int),
        ("datatype", c_short),
        ("comm", c_short),
    ]

class DumpiPackSize(Structure):
    _fields_ = [
        ("incount", c_int),
        ("datatype", c_short),
        ("comm", c_short),
        ("size", c_int),
    ]

class DumpiBarrier(Structure):
    _fields_ = [
        ("comm", c_short),
    ]

class DumpiBcast(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("root", c_int),
        ("comm", c_short),
    ]

class DumpiGather(Structure):
    _fields_ = [
        ("commrank", c_int),
        ("sendcount", c_int),
        ("sendtype", c_short),
        ("recvcount", c_int),
        ("recvtype", c_short),
        ("root", c_int),
        ("comm", c_short),
    ]

class DumpiGatherv(Structure):
    _fields_ = [
        ("commrank", c_int),
        ("commsize", c_int),
        ("sendcount", c_int),
        ("sendtype", c_short),
        ("recvcounts", POINTER(c_int)),
        ("displs", POINTER(c_int)),
        ("recvtype", c_short),
        ("root", c_int),
        ("comm", c_short),
    ]

class DumpiScatter(Structure):
    _fields_ = [
        ("commrank", c_int),
        ("sendcount", c_int),
        ("sendtype", c_short),
        ("recvcount", c_int),
        ("recvtype", c_short),
        ("root", c_int),
        ("comm", c_short),
    ]

class DumpiScatterv(Structure):
    _fields_ = [
        ("commrank", c_int),
        ("commsize", c_int),
        ("sendcounts", POINTER(c_int)),
        ("displs", POINTER(c_int)),
        ("sendtype", c_short),
        ("recvcount", c_int),
        ("recvtype", c_short),
        ("root", c_int),
        ("comm", c_short),
    ]

class DumpiAllgather(Structure):
    _fields_ = [
        ("sendcount", c_int),
        ("sendtype", c_short),
        ("recvcount", c_int),
        ("recvtype", c_short),
        ("comm", c_short),
    ]

class DumpiAllgatherv(Structure):
    _fields_ = [
        ("commsize", c_int),
        ("sendcount", c_int),
        ("sendtype", c_short),
        ("recvcounts", POINTER(c_int)),
        ("displs", POINTER(c_int)),
        ("recvtype", c_short),
        ("comm", c_short),
    ]

class DumpiAlltoall(Structure):
    _fields_ = [
        ("sendcount", c_int),
        ("sendtype", c_short),
        ("recvcount", c_int),
        ("recvtype", c_short),
        ("comm", c_short),
    ]

class DumpiAlltoallv(Structure):
    _fields_ = [
        ("commsize", c_int),
        ("sendcounts", POINTER(c_int)),
        ("senddispls", POINTER(c_int)),
        ("sendtype", c_short),
        ("recvcounts", POINTER(c_int)),
        ("recvdispls", POINTER(c_int)),
        ("recvtype", c_short),
        ("comm", c_short),
    ]

class DumpiReduce(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("op", c_short),
        ("root", c_int),
        ("comm", c_short),
    ]

class DumpiOpCreate(Structure):
    _fields_ = [
        ("commute", c_int),
        ("op", c_short),
    ]

class DumpiOpFree(Structure):
    _fields_ = [
        ("op", c_short),
    ]

class DumpiAllreduce(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("op", c_short),
        ("comm", c_short),
    ]

class DumpiReduceScatter(Structure):
    _fields_ = [
        ("commsize", c_int),
        ("recvcounts", POINTER(c_int)),
        ("datatype", c_short),
        ("op", c_short),
        ("comm", c_short),
    ]

class DumpiScan(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("op", c_short),
        ("comm", c_short),
    ]

class DumpiGroupSize(Structure):
    _fields_ = [
        ("group", c_short),
        ("size", c_int),
    ]

class DumpiGroupRank(Structure):
    _fields_ = [
        ("group", c_short),
        ("rank", c_int),
    ]

class DumpiGroupTranslateRanks(Structure):
    _fields_ = [
        ("group1", c_short),
        ("count", c_int),
        ("ranks1", POINTER(c_int)),
        ("group2", c_short),
        ("ranks2", POINTER(c_int)),
    ]

class DumpiGroupCompare(Structure):
    _fields_ = [
        ("group1", c_short),
        ("group2", c_short),
        ("result", c_byte),
    ]

class DumpiCommGroup(Structure):
    _fields_ = [
        ("comm", c_short),
        ("group", c_short),
    ]

class DumpiGroupUnion(Structure):
    _fields_ = [
        ("group1", c_short),
        ("group2", c_short),
        ("newgroup", c_short),
    ]

class DumpiGroupIntersection(Structure):
    _fields_ = [
        ("group1", c_short),
        ("group2", c_short),
        ("newgroup", c_short),
    ]

class DumpiGroupDifference(Structure):
    _fields_ = [
        ("group1", c_short),
        ("group2", c_short),
        ("newgroup", c_short),
    ]

class DumpiGroupIncl(Structure):
    _fields_ = [
        ("group", c_short),
        ("count", c_int),
        ("ranks", POINTER(c_int)),
        ("newgroup", c_short),
    ]

class DumpiGroupExcl(Structure):
    _fields_ = [
        ("group", c_short),
        ("count", c_int),
        ("ranks", POINTER(c_int)),
        ("newgroup", c_short),
    ]

class DumpiGroupRangeIncl(Structure):
    _fields_ = [
        ("group", c_short),
        ("count", c_int),
        ("ranges", POINTER(POINTER(c_int))),
        ("newgroup", c_short),
    ]

class DumpiGroupRangeExcl(Structure):
    _fields_ = [
        ("group", c_short),
        ("count", c_int),
        ("ranges", POINTER(POINTER(c_int))),
        ("newgroup", c_short),
    ]

class DumpiGroupFree(Structure):
    _fields_ = [
        ("group", c_short),
    ]

class DumpiCommSize(Structure):
    _fields_ = [
        ("comm", c_short),
        ("size", c_int),
    ]

class DumpiCommRank(Structure):
    _fields_ = [
        ("comm", c_short),
        ("rank", c_int),
    ]

class DumpiCommCompare(Structure):
    _fields_ = [
        ("comm1", c_short),
        ("comm2", c_short),
        ("result", c_byte),
    ]

class DumpiCommDup(Structure):
    _fields_ = [
        ("oldcomm", c_short),
        ("newcomm", c_short),
    ]

class DumpiCommCreate(Structure):
    _fields_ = [
        ("oldcomm", c_short),
        ("group", c_short),
        ("newcomm", c_short),
    ]

class DumpiCommSplit(Structure):
    _fields_ = [
        ("oldcomm", c_short),
        ("color", c_int),
        ("key", c_int),
        ("newcomm", c_short),
    ]

class DumpiCommFree(Structure):
    _fields_ = [
        ("comm", c_short),
    ]

class DumpiCommTestInter(Structure):
    _fields_ = [
        ("comm", c_short),
        ("inter", c_int),
    ]

class DumpiCommRemoteSize(Structure):
    _fields_ = [
        ("comm", c_short),
        ("size", c_int),
    ]

class DumpiCommRemoteGroup(Structure):
    _fields_ = [
        ("comm", c_short),
        ("group", c_short),
    ]

class DumpiIntercommCreate(Structure):
    _fields_ = [
        ("localcomm", c_short),
        ("localleader", c_int),
        ("remotecomm", c_short),
        ("remoteleader", c_int),
        ("tag", c_int),
        ("newcomm", c_short),
    ]

class DumpiIntercommMerge(Structure):
    _fields_ = [
        ("comm", c_short),
        ("hight", c_int),
        ("newcomm", c_short),
    ]

class DumpiKeyvalCreate(Structure):
    _fields_ = [
        ("key", c_short),
    ]

class DumpiKeyvalFree(Structure):
    _fields_ = [
        ("key", c_short),
    ]

class DumpiAttrPut(Structure):
    _fields_ = [
        ("comm", c_short),
        ("key", c_int),
    ]

class DumpiAttrGet(Structure):
    _fields_ = [
        ("comm", c_short),
        ("key", c_int),
        ("flag", c_int),
    ]

class DumpiAttrDelete(Structure):
    _fields_ = [
        ("comm", c_short),
        ("key", c_int),
    ]

class DumpiTopoTest(Structure):
    _fields_ = [
        ("comm", c_short),
        ("topology", c_short),
    ]

class DumpiCartCreate(Structure):
    _fields_ = [
        ("oldcomm", c_short),
        ("ndim", c_int),
        ("dims", POINTER(c_int)),
        ("periods", POINTER(c_int)),
        ("reorder", c_int),
        ("newcomm", c_short),
    ]

class DumpiDimsCreate(Structure):
    _fields_ = [
        ("nodes", c_int),
        ("ndim", c_int),
        ("pos_in", POINTER(c_int)),
        ("pos_out", POINTER(c_int)),
    ]

class DumpiGraphCreate(Structure):
    _fields_ = [
        ("numedges", c_int),
        ("oldcomm", c_short),
        ("nodes", c_int),
        ("index", POINTER(c_int)),
        ("edges", POINTER(c_int)),
        ("reoder", c_int),
        ("newcomm", c_short),
    ]

class DumpiGraphdimsGet(Structure):
    _fields_ = [
        ("comm", c_short),
        ("nodes", c_int),
        ("edges", c_int),
    ]

class DumpiGraphGet(Structure):
    _fields_ = [
        ("totedges", c_int),
        ("totnodes", c_int),
        ("comm", c_short),
        ("maxindex", c_int),
        ("maxedges", c_int),
        ("index", POINTER(c_int)),
        ("edges", POINTER(c_int)),
    ]

class DumpiCartdimGet(Structure):
    _fields_ = [
        ("comm", c_short),
        ("ndim", c_int),
    ]

class DumpiCartGet(Structure):
    _fields_ = [
        ("ndim", c_int),
        ("comm", c_short),
        ("maxdims", c_int),
        ("dims", POINTER(c_int)),
        ("periods", POINTER(c_int)),
        ("coords", POINTER(c_int)),
    ]

class DumpiCartRank(Structure):
    _fields_ = [
        ("ndim", c_int),
        ("comm", c_short),
        ("coords", POINTER(c_int)),
        ("rank", c_int),
    ]

class DumpiCartCoords(Structure):
    _fields_ = [
        ("ndim", c_int),
        ("comm", c_short),
        ("rank", c_int),
        ("maxdims", c_int),
        ("coords", POINTER(c_int)),
    ]

class DumpiGraphNeighborsCount(Structure):
    _fields_ = [
        ("comm", c_short),
        ("rank", c_int),
        ("nneigh", c_int),
    ]

class DumpiGraphNeighbors(Structure):
    _fields_ = [
        ("nneigh", c_int),
        ("comm", c_short),
        ("rank", c_int),
        ("maxneighbors", c_int),
        ("neighbors", POINTER(c_int)),
    ]

class DumpiCartShift(Structure):
    _fields_ = [
        ("comm", c_short),
        ("direction", c_int),
        ("displ", c_int),
        ("source", c_int),
        ("dest", c_int),
    ]

class DumpiCartSub(Structure):
    _fields_ = [
        ("ndim", c_int),
        ("oldcomm", c_short),
        ("remain_dim", POINTER(c_int)),
        ("newcomm", c_short),
    ]

class DumpiCartMap(Structure):
    _fields_ = [
        ("comm", c_short),
        ("ndim", c_int),
        ("dims", POINTER(c_int)),
        ("period", POINTER(c_int)),
        ("newrank", c_int),
    ]

class DumpiGraphMap(Structure):
    _fields_ = [
        ("numedges", c_int),
        ("comm", c_short),
        ("nodes", c_int),
        ("index", POINTER(c_int)),
        ("edges", POINTER(c_int)),
        ("newrank", c_int),
    ]

class DumpiGetProcessorName(Structure):
    _fields_ = [
        ("name", c_char_p),
        ("resultlen", c_int),
    ]

class DumpiGetVersion(Structure):
    _fields_ = [
        ("version", c_int),
        ("subversion", c_int),
    ]

class DumpiErrhandlerCreate(Structure):
    _fields_ = [
        ("errhandler", c_short),
    ]

class DumpiErrhandlerSet(Structure):
    _fields_ = [
        ("comm", c_short),
        ("errhandler", c_short),
    ]

class DumpiErrhandlerGet(Structure):
    _fields_ = [
        ("comm", c_short),
        ("errhandler", c_short),
    ]

class DumpiErrhandlerFree(Structure):
    _fields_ = [
        ("errhandler", c_short),
    ]

class DumpiErrorString(Structure):
    _fields_ = [
        ("errorcode", c_int),
        ("errorstring", c_char_p),
        ("resultlen", c_int),
    ]

class DumpiErrorClass(Structure):
    _fields_ = [
        ("errorcode", c_int),
        ("errorclass", c_int),
    ]

class DumpiWtime(Structure):
    _fields_ = [
        ("psec", c_ulonglong),
    ]

class DumpiWtick(Structure):
    _fields_ = [
        ("psec", c_ulonglong),
    ]

class DumpiInit(Structure):
    _fields_ = [
        ("argc", c_int),
        ("argv", POINTER(POINTER(c_char))),
    ]

class DumpiFinalize(Structure):
    _fields_ = [
        ("dummy", c_int),
    ]

class DumpiInitialized(Structure):
    _fields_ = [
        ("result", c_int),
    ]

class DumpiAbort(Structure):
    _fields_ = [
        ("comm", c_short),
        ("errorcode", c_int),
    ]

class DumpiClosePort(Structure):
    _fields_ = [
        ("portname", c_char_p),
    ]

class DumpiCommAccept(Structure):
    _fields_ = [
        ("portname", c_char_p),
        ("info", c_short),
        ("root", c_int),
        ("oldcomm", c_short),
        ("newcomm", c_short),
    ]

class DumpiCommConnect(Structure):
    _fields_ = [
        ("portname", c_char_p),
        ("info", c_short),
        ("root", c_int),
        ("oldcomm", c_short),
        ("newcomm", c_short),
    ]

class DumpiCommDisconnect(Structure):
    _fields_ = [
        ("comm", c_short),
    ]

class DumpiCommGetParent(Structure):
    _fields_ = [
        ("parent", c_short),
    ]

class DumpiCommJoin(Structure):
    _fields_ = [
        ("fd", c_int),
        ("comm", c_short),
    ]

class DumpiCommSpawn(Structure):
    _fields_ = [
        ("oldcommrank", c_int),
        ("command", c_char_p),
        ("argv", POINTER(POINTER(c_char))),
        ("maxprocs", c_int),
        ("info", c_short),
        ("root", c_int),
        ("oldcomm", c_short),
        ("newcomm", c_short),
        ("errcodes", c_int),
    ]

class DumpiCommSpawnMultiple(Structure):
    _fields_ = [
        ("totprocs", c_int),
        ("oldcommrank", c_int),
        ("count", c_int),
        ("commands", POINTER(c_char_p)),
        ("argvs", POINTER(POINTER(POINTER(c_char)))),
        ("maxprocs", POINTER(c_int)),
        ("info", POINTER(c_short)),
        ("root", c_int),
        ("oldcomm", c_short),
        ("newcomm", c_short),
        ("errcodes", c_int),
    ]

class DumpiLookupName(Structure):
    _fields_ = [
        ("servicename", c_char_p),
        ("info", c_short),
        ("portname", c_char_p),
    ]

class DumpiOpenPort(Structure):
    _fields_ = [
        ("info", c_short),
        ("portname", c_char_p),
    ]

class DumpiPublishName(Structure):
    _fields_ = [
        ("servicename", c_char_p),
        ("info", c_short),
        ("portname", c_char_p),
    ]

class DumpiUnpublishName(Structure):
    _fields_ = [
        ("servicename", c_char_p),
        ("info", c_short),
        ("portname", c_char_p),
    ]

class DumpiAccumulate(Structure):
    _fields_ = [
        ("origincount", c_int),
        ("origintype", c_short),
        ("targetrank", c_int),
        ("targetdisp", c_int),
        ("targetcount", c_int),
        ("targettype", c_short),
        ("op", c_short),
        ("win", c_short),
    ]

class DumpiGet(Structure):
    _fields_ = [
        ("origincount", c_int),
        ("origintype", c_short),
        ("targetrank", c_int),
        ("targetdisp", c_int),
        ("targetcount", c_int),
        ("targettype", c_short),
        ("win", c_short),
    ]

class DumpiPut(Structure):
    _fields_ = [
        ("origincount", c_int),
        ("origintype", c_short),
        ("targetrank", c_int),
        ("targetdisp", c_int),
        ("targetcount", c_int),
        ("targettype", c_short),
        ("win", c_short),
    ]

class DumpiWinComplete(Structure):
    _fields_ = [
        ("win", c_short),
    ]

class DumpiWinCreate(Structure):
    _fields_ = [
        ("size", c_int),
        ("dispunit", c_int),
        ("info", c_short),
        ("comm", c_short),
        ("win", c_short),
    ]

class DumpiWinFence(Structure):
    _fields_ = [
        ("assertion", c_short),
        ("win", c_short),
    ]

class DumpiWinFree(Structure):
    _fields_ = [
        ("win", c_short),
    ]

class DumpiWinGetGroup(Structure):
    _fields_ = [
        ("win", c_short),
        ("group", c_short),
    ]

class DumpiWinLock(Structure):
    _fields_ = [
        ("locktype", c_short),
        ("winrank", c_int),
        ("assertion", c_short),
        ("win", c_short),
    ]

class DumpiWinPost(Structure):
    _fields_ = [
        ("group", c_short),
        ("assertion", c_short),
        ("win", c_short),
    ]

class DumpiWinStart(Structure):
    _fields_ = [
        ("group", c_short),
        ("assertion", c_short),
        ("win", c_short),
    ]

class DumpiWinTest(Structure):
    _fields_ = [
        ("win", c_short),
        ("flag", c_int),
    ]

class DumpiWinUnlock(Structure):
    _fields_ = [
        ("winrank", c_int),
        ("win", c_short),
    ]

class DumpiWinWait(Structure):
    _fields_ = [
        ("win", c_short),
    ]

class DumpiAlltoallw(Structure):
    _fields_ = [
        ("commsize", c_int),
        ("sendcounts", POINTER(c_int)),
        ("senddispls", POINTER(c_int)),
        ("sendtypes", POINTER(c_short)),
        ("recvcounts", POINTER(c_int)),
        ("recvdispls", POINTER(c_int)),
        ("recvtypes", POINTER(c_short)),
        ("comm", c_short),
    ]

class DumpiExscan(Structure):
    _fields_ = [
        ("count", c_int),
        ("datatype", c_short),
        ("op", c_short),
        ("comm", c_short),
    ]

class DumpiAddErrorClass(Structure):
    _fields_ = [
        ("errorclass", c_int),
    ]

class DumpiAddErrorCode(Structure):
    _fields_ = [
        ("errorclass", c_int),
        ("errorcode", c_int),
    ]

class DumpiAddErrorString(Structure):
    _fields_ = [
        ("errorcode", c_int),
        ("errorstring", c_char_p),
    ]

class DumpiCommCallErrhandler(Structure):
    _fields_ = [
        ("comm", c_short),
        ("errorcode", c_int),
    ]

class DumpiCommCreateKeyval(Structure):
    _fields_ = [
        ("keyval", c_short),
    ]

class DumpiCommDeleteAttr(Structure):
    _fields_ = [
        ("comm", c_short),
        ("keyval", c_short),
    ]

class DumpiCommFreeKeyval(Structure):
    _fields_ = [
        ("keyval", c_short),
    ]

class DumpiCommGetAttr(Structure):
    _fields_ = [
        ("comm", c_short),
        ("keyval", c_short),
        ("flag", c_int),
    ]

class DumpiCommGetName(Structure):
    _fields_ = [
        ("comm", c_short),
        ("name", c_char_p),
        ("resultlen", c_int),
    ]

class DumpiCommSetAttr(Structure):
    _fields_ = [
        ("comm", c_short),
        ("keyval", c_short),
    ]

class DumpiCommSetName(Structure):
    _fields_ = [
        ("comm", c_short),
        ("name", c_char_p),
    ]

class DumpiFileCallErrhandler(Structure):
    _fields_ = [
        ("file", c_short),
        ("errorcode", c_int),
    ]

class DumpiGrequestComplete(Structure):
    _fields_ = [
        ("request", c_int),
    ]

class DumpiGrequestStart(Structure):
    _fields_ = [
        ("request", c_int),
    ]

class DumpiInitThread(Structure):
    _fields_ = [
        ("argc", c_int),
        ("argv", POINTER(POINTER(c_char))),
        ("tl_required", c_short),
        ("tl_provided", c_short),
    ]

class DumpiIsThreadMain(Structure):
    _fields_ = [
        ("flag", c_int),
    ]

class DumpiQueryThread(Structure):
    _fields_ = [
        ("tl_supported", c_short),
    ]

class DumpiStatusSetCancelled(Structure):
    _fields_ = [
        ("status", POINTER(DumpiStatus)),
        ("flag", c_int),
    ]

class DumpiStatusSetElements(Structure):
    _fields_ = [
        ("status", POINTER(DumpiStatus)),
        ("datatype", c_short),
        ("count", c_int),
    ]

class DumpiTypeCreateKeyval(Structure):
    _fields_ = [
        ("keyval", c_short),
    ]

class DumpiTypeDeleteAttr(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("keyval", c_short),
    ]

class DumpiTypeDup(Structure):
    _fields_ = [
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeFreeKeyval(Structure):
    _fields_ = [
        ("keyval", c_short),
    ]

class DumpiTypeGetAttr(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("keyval", c_short),
        ("flag", c_int),
    ]

class DumpiTypeGetContents(Structure):
    _fields_ = [
        ("numdatatypes", c_int),
        ("numaddresses", c_int),
        ("numintegers", c_int),
        ("datatype", c_short),
        ("maxintegers", c_int),
        ("maxaddresses", c_int),
        ("maxdatatyeps", c_int),
        ("arrintegers", POINTER(c_int)),
        ("arraddresses", POINTER(c_int)),
        ("arrdatatypes", POINTER(c_short)),
    ]

class DumpiTypeGetEnvelope(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("numintegers", c_int),
        ("numaddresses", c_int),
        ("numdatatypes", c_int),
        ("combiner", c_short),
    ]

class DumpiTypeGetName(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("name", c_char_p),
        ("resultlen", c_int),
    ]

class DumpiTypeSetAttr(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("keyval", c_short),
    ]

class DumpiTypeSetName(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("name", c_char_p),
    ]

class DumpiTypeMatchSize(Structure):
    _fields_ = [
        ("typeclass", c_short),
        ("size", c_int),
        ("datatype", c_short),
    ]

class DumpiWinCallErrhandler(Structure):
    _fields_ = [
        ("win", c_short),
        ("errorcode", c_int),
    ]

class DumpiWinCreateKeyval(Structure):
    _fields_ = [
        ("keyval", c_short),
    ]

class DumpiWinDeleteAttr(Structure):
    _fields_ = [
        ("win", c_short),
        ("keyval", c_short),
    ]

class DumpiWinFreeKeyval(Structure):
    _fields_ = [
        ("keyval", c_short),
    ]

class DumpiWinGetAttr(Structure):
    _fields_ = [
        ("win", c_short),
        ("keyval", c_short),
        ("flag", c_int),
    ]

class DumpiWinGetName(Structure):
    _fields_ = [
        ("win", c_short),
        ("name", c_char_p),
        ("resultlen", c_int),
    ]

class DumpiWinSetAttr(Structure):
    _fields_ = [
        ("win", c_short),
        ("keyval", c_short),
    ]

class DumpiWinSetName(Structure):
    _fields_ = [
        ("win", c_short),
        ("name", c_char_p),
    ]

class DumpiAllocMem(Structure):
    _fields_ = [
        ("size", c_int),
        ("info", c_short),
    ]

class DumpiCommCreateErrhandler(Structure):
    _fields_ = [
        ("errhandler", c_short),
    ]

class DumpiCommGetErrhandler(Structure):
    _fields_ = [
        ("comm", c_short),
        ("errhandler", c_short),
    ]

class DumpiCommSetErrhandler(Structure):
    _fields_ = [
        ("comm", c_short),
        ("errhandler", c_short),
    ]

class DumpiFileCreateErrhandler(Structure):
    _fields_ = [
        ("errhandler", c_short),
    ]

class DumpiFileGetErrhandler(Structure):
    _fields_ = [
        ("file", c_short),
        ("errhandler", c_short),
    ]

class DumpiFileSetErrhandler(Structure):
    _fields_ = [
        ("file", c_short),
        ("errhandler", c_short),
    ]

class DumpiFinalized(Structure):
    _fields_ = [
        ("flag", c_int),
    ]

class DumpiFreeMem(Structure):
    _fields_ = [
        ("dummy", c_int),
    ]

class DumpiGetAddress(Structure):
    _fields_ = [
        ("address", c_int),
    ]

class DumpiInfoCreate(Structure):
    _fields_ = [
        ("info", c_short),
    ]

class DumpiInfoDelete(Structure):
    _fields_ = [
        ("info", c_short),
        ("key", c_char_p),
    ]

class DumpiInfoDup(Structure):
    _fields_ = [
        ("oldinfo", c_short),
        ("newinfo", c_short),
    ]

class DumpiInfoFree(Structure):
    _fields_ = [
        ("info", c_short),
    ]

class DumpiInfoGet(Structure):
    _fields_ = [
        ("info", c_short),
        ("key", c_char_p),
        ("valuelength", c_int),
        ("value", c_char_p),
        ("flag", c_int),
    ]

class DumpiInfoGetNkeys(Structure):
    _fields_ = [
        ("info", c_short),
        ("nkeys", c_int),
    ]

class DumpiInfoGetNthkey(Structure):
    _fields_ = [
        ("info", c_short),
        ("n", c_int),
        ("key", c_char_p),
    ]

class DumpiInfoGetValuelen(Structure):
    _fields_ = [
        ("info", c_short),
        ("key", c_char_p),
        ("valuelen", c_int),
        ("flag", c_int),
    ]

class DumpiInfoSet(Structure):
    _fields_ = [
        ("info", c_short),
        ("key", c_char_p),
        ("value", c_char_p),
    ]

class DumpiPackExternal(Structure):
    _fields_ = [
        ("datarep", POINTER(c_char)),
        ("incount", c_int),
        ("intype", c_short),
        ("outcount", c_int),
        ("pos_in", c_int),
        ("pos_out", c_int),
    ]

class DumpiPackExternalSize(Structure):
    _fields_ = [
        ("datarep", POINTER(c_char)),
        ("incount", c_int),
        ("datatype", c_short),
        ("size", c_int),
    ]

class DumpiRequestGetStatus(Structure):
    _fields_ = [
        ("request", c_int),
        ("flag", c_int),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiTypeCreateDarray(Structure):
    _fields_ = [
        ("size", c_int),
        ("rank", c_int),
        ("ndims", c_int),
        ("gsizes", POINTER(c_int)),
        ("distribs", POINTER(c_short)),
        ("dargs", POINTER(c_int)),
        ("psizes", POINTER(c_int)),
        ("order", c_byte),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeCreateHindexed(Structure):
    _fields_ = [
        ("count", c_int),
        ("blocklengths", POINTER(c_int)),
        ("displacements", POINTER(c_int)),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeCreateHvector(Structure):
    _fields_ = [
        ("count", c_int),
        ("blocklength", c_int),
        ("stride", c_int),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeCreateIndexedBlock(Structure):
    _fields_ = [
        ("count", c_int),
        ("blocklength", c_int),
        ("displacements", POINTER(c_int)),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeCreateResized(Structure):
    _fields_ = [
        ("oldtype", c_short),
        ("lb", c_int),
        ("extent", c_int),
        ("newtype", c_short),
    ]

class DumpiTypeCreateStruct(Structure):
    _fields_ = [
        ("count", c_int),
        ("blocklengths", POINTER(c_int)),
        ("displacements", POINTER(c_int)),
        ("oldtypes", POINTER(c_short)),
        ("newtype", c_short),
    ]

class DumpiTypeCreateSubarray(Structure):
    _fields_ = [
        ("ndims", c_int),
        ("sizes", POINTER(c_int)),
        ("subsizes", POINTER(c_int)),
        ("starts", POINTER(c_int)),
        ("order", c_byte),
        ("oldtype", c_short),
        ("newtype", c_short),
    ]

class DumpiTypeGetExtent(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("lb", c_int),
        ("extent", c_int),
    ]

class DumpiTypeGetTrueExtent(Structure):
    _fields_ = [
        ("datatype", c_short),
        ("lb", c_int),
        ("extent", c_int),
    ]

class DumpiUnpackExternal(Structure):
    _fields_ = [
        ("datarep", POINTER(c_char)),
        ("insize", c_int),
        ("pos_in", c_int),
        ("pos_out", c_int),
        ("outcount", c_int),
        ("datatype", c_short),
    ]

class DumpiWinCreateErrhandler(Structure):
    _fields_ = [
        ("errhandler", c_short),
    ]

class DumpiWinGetErrhandler(Structure):
    _fields_ = [
        ("win", c_short),
        ("errhandler", c_short),
    ]

class DumpiWinSetErrhandler(Structure):
    _fields_ = [
        ("win", c_short),
        ("errhandler", c_short),
    ]

class DumpiFileOpen(Structure):
    _fields_ = [
        ("comm", c_short),
        ("filename", c_char_p),
        ("amode", c_short),
        ("info", c_short),
        ("file", c_short),
    ]

class DumpiFileClose(Structure):
    _fields_ = [
        ("file", c_short),
    ]

class DumpiFileDelete(Structure):
    _fields_ = [
        ("filename", c_char_p),
        ("info", c_short),
    ]

class DumpiFileSetSize(Structure):
    _fields_ = [
        ("file", c_short),
        ("size", c_longlong),
    ]

class DumpiFilePreallocate(Structure):
    _fields_ = [
        ("file", c_short),
        ("size", c_longlong),
    ]

class DumpiFileGetSize(Structure):
    _fields_ = [
        ("file", c_short),
        ("size", c_longlong),
    ]

class DumpiFileGetGroup(Structure):
    _fields_ = [
        ("file", c_short),
        ("group", c_short),
    ]

class DumpiFileGetAmode(Structure):
    _fields_ = [
        ("file", c_short),
        ("amode", c_short),
    ]

class DumpiFileSetInfo(Structure):
    _fields_ = [
        ("file", c_short),
        ("info", c_short),
    ]

class DumpiFileGetInfo(Structure):
    _fields_ = [
        ("file", c_short),
        ("info", c_short),
    ]

class DumpiFileSetView(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("hosttype", c_short),
        ("filetype", c_short),
        ("datarep", POINTER(c_char)),
        ("info", c_short),
    ]

class DumpiFileGetView(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("hosttype", c_short),
        ("filetype", c_short),
        ("datarep", POINTER(c_char)),
    ]

class DumpiFileReadAt(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileReadAtAll(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileWriteAt(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileWriteAtAll(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileIreadAt(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("count", c_int),
        ("datatype", c_short),
        ("request", c_int),
    ]

class DumpiFileIwriteAt(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("count", c_int),
        ("datatype", c_short),
        ("request", c_int),
    ]

class DumpiFileRead(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileReadAll(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileWrite(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileWriteAll(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileIread(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("request", c_int),
    ]

class DumpiFileIwrite(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("request", c_int),
    ]

class DumpiFileSeek(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("whence", c_short),
    ]

class DumpiFileGetPosition(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
    ]

class DumpiFileGetByteOffset(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("bytes", c_longlong),
    ]

class DumpiFileReadShared(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileWriteShared(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileIreadShared(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("request", c_int),
    ]

class DumpiFileIwriteShared(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("request", c_int),
    ]

class DumpiFileReadOrdered(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileWriteOrdered(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileSeekShared(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("whence", c_short),
    ]

class DumpiFileGetPositionShared(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
    ]

class DumpiFileReadAtAllBegin(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("count", c_int),
        ("datatype", c_short),
    ]

class DumpiFileReadAtAllEnd(Structure):
    _fields_ = [
        ("file", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileWriteAtAllBegin(Structure):
    _fields_ = [
        ("file", c_short),
        ("offset", c_longlong),
        ("count", c_int),
        ("datatype", c_short),
    ]

class DumpiFileWriteAtAllEnd(Structure):
    _fields_ = [
        ("file", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileReadAllBegin(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
    ]

class DumpiFileReadAllEnd(Structure):
    _fields_ = [
        ("file", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileWriteAllBegin(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
    ]

class DumpiFileWriteAllEnd(Structure):
    _fields_ = [
        ("file", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileReadOrderedBegin(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
    ]

class DumpiFileReadOrderedEnd(Structure):
    _fields_ = [
        ("file", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileWriteOrderedBegin(Structure):
    _fields_ = [
        ("file", c_short),
        ("count", c_int),
        ("datatype", c_short),
    ]

class DumpiFileWriteOrderedEnd(Structure):
    _fields_ = [
        ("file", c_short),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpiFileGetTypeExtent(Structure):
    _fields_ = [
        ("file", c_short),
        ("datatype", c_short),
        ("extent", c_int),
    ]

class DumpiRegisterDatarep(Structure):
    _fields_ = [
        ("name", c_char_p),
    ]

class DumpiFileSetAtomicity(Structure):
    _fields_ = [
        ("file", c_short),
        ("flag", c_int),
    ]

class DumpiFileGetAtomicity(Structure):
    _fields_ = [
        ("file", c_short),
        ("flag", c_int),
    ]

class DumpiFileSync(Structure):
    _fields_ = [
        ("file", c_short),
    ]

class DumpioTest(Structure):
    _fields_ = [
        ("request", c_int),
        ("flag", c_int),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpioWait(Structure):
    _fields_ = [
        ("request", c_int),
        ("status", POINTER(DumpiStatus)),
    ]

class DumpioTestAll(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("flag", c_int),
        ("statuses", POINTER(DumpiStatus)),
    ]

class DumpioWaitAll(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("statuses", POINTER(DumpiStatus)),
    ]

class DumpioTestAny(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("index", c_int),
        ("flag", c_int),
        ("statuses", POINTER(DumpiStatus)),
    ]

class DumpioWaitAny(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("index", c_int),
        ("statuses", POINTER(DumpiStatus)),
    ]

class DumpioWaitSome(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("outcount", c_int),
        ("indices", POINTER(c_int)),
        ("statuses", POINTER(DumpiStatus)),
    ]

class DumpioTestSome(Structure):
    _fields_ = [
        ("count", c_int),
        ("requests", POINTER(c_int)),
        ("outcount", c_int),
        ("indices", POINTER(c_int)),
        ("statuses", POINTER(DumpiStatus)),
    ]
